@@ -38,9 +38,13 @@ logLik.gllvm <- function(object, ...)
     object$params$sigmaB <- object$params$sigmaB[lower.tri(object$params$sigmaB, diag = TRUE)]
   }
   if(object$quadratic=="LV"){
-    object$params$theta[-1,-c(1:object$num.lv)]<-NA
+    object$params$theta[-1,-c(1:(object$num.lv+object$num.lv.c))]<-NA
   }
-  attributes(logL)$df <- length(unlist(object$params)[!is.na(unlist(object$params))]) - object$num.lv * (object$num.lv - 1) / 2
+  if(object$num.lv>0|object$num.RR>0|object$num.lv.c>0){
+    object$params$theta[object$params$theta==0|object$params$theta==1]<-NA  
+  }
+  
+  attributes(logL)$df <- length(unlist(object$params)[!is.na(unlist(object$params))])
   attributes(logL)$nobs <- dim(object$y)[1]
   class(logL) <- "logLik"
   return(logL)
